﻿namespace RavenDB_RegisterIdConvention_FailingTest
{
    using System;
    using System.Collections.Generic;

    using NUnit.Framework;

    using Raven.Client;
    using Raven.Client.Embedded;

    /// <summary>
    /// Represents the unit tests for register id convention.
    /// </summary>
    [TestFixture]
    public class RegisterIdConventionTest
    {
        private IDocumentStore _documentStore;

        /// <summary>
        /// Initializes each test.
        /// </summary>
        [SetUp]
        public void Init()
        {
            if ((null != _documentStore) && !_documentStore.WasDisposed)
            {
                return;
            }

            _documentStore = new EmbeddableDocumentStore { DataDirectory = "Data", UseEmbeddedHttpServer = false };
            _documentStore.Initialize();
            _documentStore.Conventions.RegisterIdConvention<MyClassWithIdProperty>((dbName, commands, myClassWithIdProperty) => "myClasses/" + myClassWithIdProperty.Reference);
            _documentStore.Conventions.RegisterIdConvention<MyClassWithoutIdProperty>((dbName, commands, myClassWithIdProperty) => "myClassesNoId/" + myClassWithIdProperty.Reference);
        }

        /// <summary>
        /// Finalises each test.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            _documentStore.Dispose();
        }

        /// <summary>
        /// Tests that the stored id is as per registered convention when given object has a property called id.
        /// </summary>
        [Test]
        public void TestStoredIdIsCorrectWithConventionAndIdProperty()
        {
            MyClassWithIdProperty myClassWithIdProperty = new MyClassWithIdProperty { Id = 1, Reference = Guid.NewGuid().ToString(), CreationTime = DateTime.Now };
            string supposedId = "myClasses/" + myClassWithIdProperty.Reference;

            string documentId;
            using (IDocumentSession documentSession = _documentStore.OpenSession())
            {
                documentSession.Store(myClassWithIdProperty);
                documentSession.SaveChanges();
                documentId = documentSession.Advanced.GetDocumentId(myClassWithIdProperty);
            }

            Assert.AreEqual(supposedId, documentId, "Retrieved id was incorrect.");
        }

        /// <summary>
        /// Tests that the stored id is as per registered convention when given object does not have a property called id.
        /// </summary>
        [Test]
        public void TestStoredIdIsCorrectWithConventionAndWithoutIdProperty()
        {
            MyClassWithoutIdProperty myClassWithIdProperty = new MyClassWithoutIdProperty { Reference = Guid.NewGuid().ToString(), CreationTime = DateTime.Now };
            string supposedId = "myClassesNoId/" + myClassWithIdProperty.Reference;

            string documentId;
            using (IDocumentSession documentSession = _documentStore.OpenSession())
            {
                documentSession.Store(myClassWithIdProperty);
                documentSession.SaveChanges();
                documentId = documentSession.Advanced.GetDocumentId(myClassWithIdProperty);
            }

            Assert.AreEqual(supposedId, documentId, "Retrieved id was incorrect.");
        }
    }
}
